@@ -113,17 +113,20 @@ static void 		gst_basic_scheduler_reset 		(GstScheduler *sched);
 static void		gst_basic_scheduler_add_element		(GstScheduler *sched, GstElement *element);
 static void     	gst_basic_scheduler_remove_element	(GstScheduler *sched, GstElement *element);
 static GstElementStateReturn  
-			gst_basic_scheduler_state_transition	(GstScheduler *sched, GstElement *element, gint transition);
+			gst_basic_scheduler_state_transition	(GstScheduler *sched, GstElement *element,
+								 gint transition);
 static void 		gst_basic_scheduler_lock_element 	(GstScheduler *sched, GstElement *element);
 static void 		gst_basic_scheduler_unlock_element 	(GstScheduler *sched, GstElement *element);
-static void 		gst_basic_scheduler_yield 		(GstScheduler *sched, GstElement *element);
+static gboolean		gst_basic_scheduler_yield 		(GstScheduler *sched, GstElement *element);
 static gboolean		gst_basic_scheduler_interrupt 		(GstScheduler *sched, GstElement *element);
 static void 		gst_basic_scheduler_error	 	(GstScheduler *sched, GstElement *element);
-static void     	gst_basic_scheduler_pad_connect		(GstScheduler *sched, GstPad *srcpad, GstPad *sinkpad);
-static void     	gst_basic_scheduler_pad_disconnect 	(GstScheduler *sched, GstPad *srcpad, GstPad *sinkpad);
+static void     	gst_basic_scheduler_pad_link		(GstScheduler *sched, GstPad *srcpad,
+								 GstPad *sinkpad);
+static void     	gst_basic_scheduler_pad_unlink 		(GstScheduler *sched, GstPad *srcpad,
+								 GstPad *sinkpad);
 static GstPad*  	gst_basic_scheduler_pad_select 		(GstScheduler *sched, GList *padlist);
 static GstClockReturn	gst_basic_scheduler_clock_wait	 	(GstScheduler *sched, GstElement *element,
-								 GstClock *clock, GstClockTime time, GstClockTimeDiff *jitter);
+								 GstClockID id, GstClockTimeDiff *jitter);
 static GstSchedulerState
 			gst_basic_scheduler_iterate    		(GstScheduler *sched);
 
@@ -209,8 +212,8 @@ gst_basic_scheduler_class_init (GstBasicSchedulerClass * klass)
   gstscheduler_class->yield	 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_yield);
   gstscheduler_class->interrupt 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_interrupt);
   gstscheduler_class->error	 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_error);
-  gstscheduler_class->pad_connect 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_pad_connect);
-  gstscheduler_class->pad_disconnect 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_pad_disconnect);
+  gstscheduler_class->pad_link 		= GST_DEBUG_FUNCPTR (gst_basic_scheduler_pad_link);
+  gstscheduler_class->pad_unlink 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_pad_unlink);
   gstscheduler_class->pad_select	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_pad_select);
   gstscheduler_class->clock_wait	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_clock_wait);
   gstscheduler_class->iterate 		= GST_DEBUG_FUNCPTR (gst_basic_scheduler_iterate);
@@ -319,7 +322,7 @@ gst_basic_scheduler_chain_wrapper (int argc, char *argv[])
       realpad = GST_REAL_PAD_CAST (pad);
 
       if (GST_RPAD_DIRECTION (realpad) == GST_PAD_SINK && 
-	  GST_PAD_IS_USABLE (realpad)) {
+	  GST_PAD_IS_LINKED (realpad)) {
 	GstBuffer *buf;
 
 	GST_DEBUG (GST_CAT_DATAFLOW, "pulling data from %s:%s", name, 
@@ -469,7 +472,8 @@ gst_basic_scheduler_select_proxy (GstPad * pad, GstBuffer * buf)
   GST_RPAD_BUFPEN (GST_RPAD_PEER (pad)) = buf;
   GST_DEBUG (GST_CAT_DATAFLOW, "switching to %p",
 	     GST_ELEMENT_THREADSTATE (parent));
-  parent->select_pad = pad;
+  /* FIXME temporarily diabled */
+  /* parent->select_pad = pad; */
 
   do_element_switch (parent);
   
@@ -504,7 +508,7 @@ gst_basic_scheduler_gethandler_proxy (GstPad * pad)
       GST_DEBUG (GST_CAT_DATAFLOW, "new pad in mid-switch!");
       pad = (GstPad *) GST_RPAD_PEER (peer);
       if (!pad) {
-	gst_element_error (parent, "pad unconnected");
+	gst_element_error (parent, "pad unlinked");
       }
       parent = GST_PAD_PARENT (pad);
       peer = GST_RPAD_PEER (pad);
@@ -747,14 +751,17 @@ gst_basic_scheduler_chain_destroy (GstSchedulerChain * chain)
 static void
 gst_basic_scheduler_chain_add_element (GstSchedulerChain * chain, GstElement * element)
 {
-  GST_INFO (GST_CAT_SCHEDULING, "adding element \"%s\" to chain %p", GST_ELEMENT_NAME (element),
-	    chain);
-
   /* set the sched pointer for the element */
   element->sched = GST_SCHEDULER (chain->sched);
 
-  /* add the element to the list of 'disabled' elements */
-  chain->disabled = g_list_prepend (chain->disabled, element);
+  /* add the element to either the main list or the disabled list */
+  if (GST_STATE(element) == GST_STATE_PLAYING) {
+    GST_INFO (GST_CAT_SCHEDULING, "adding element \"%s\" to chain %p enabled", GST_ELEMENT_NAME (element),chain);
+    chain->elements = g_list_prepend (chain->elements, element);
+  } else {
+    GST_INFO (GST_CAT_SCHEDULING, "adding element \"%s\" to chain %p disabled", GST_ELEMENT_NAME (element),chain);
+    chain->disabled = g_list_prepend (chain->disabled, element);
+  }
   chain->num_elements++;
 
   /* notify the scheduler that something changed */
@@ -929,13 +936,24 @@ gst_basic_scheduler_find_chain (GstBasicScheduler * sched, GstElement * element)
 }
 
 static void
-gst_basic_scheduler_chain_recursive_add (GstSchedulerChain * chain, GstElement * element)
+gst_basic_scheduler_chain_recursive_add (GstSchedulerChain * chain, GstElement * element, gboolean remove)
 {
   GList *pads;
   GstPad *pad;
   GstElement *peerelement;
+  GstSchedulerChain *prevchain;
 
-  /* add the element to the chain */
+  /* check to see if it's in a chain already */
+  prevchain = gst_basic_scheduler_find_chain (chain->sched, element);
+  /* if it's already in another chain, either remove or punt */
+  if (prevchain != NULL) {
+    if (remove == TRUE)
+      gst_basic_scheduler_chain_remove_element (prevchain, element);
+    else
+      return;
+  }
+
+  /* add it to this one */
   gst_basic_scheduler_chain_add_element (chain, element);
 
   GST_DEBUG (GST_CAT_SCHEDULING, "recursing on element \"%s\"", GST_ELEMENT_NAME (element));
@@ -952,12 +970,9 @@ gst_basic_scheduler_chain_recursive_add (GstSchedulerChain * chain, GstElement *
       GST_DEBUG (GST_CAT_SCHEDULING, "has peer %s:%s", GST_DEBUG_PAD_NAME (GST_PAD_PEER (pad)));
       peerelement = GST_PAD_PARENT (GST_PAD_PEER (pad));
       if (GST_ELEMENT_SCHED (GST_PAD_PARENT (pad)) == GST_ELEMENT_SCHED (peerelement)) {
-	GST_DEBUG (GST_CAT_SCHEDULING, "peer \"%s\" is valid for same chain",
+        GST_DEBUG (GST_CAT_SCHEDULING, "peer \"%s\" is valid for same chain",
 		   GST_ELEMENT_NAME (peerelement));
-	/* if it's not already in a chain, add it to this one */
-	if (gst_basic_scheduler_find_chain (chain->sched, peerelement) == NULL) {
-	  gst_basic_scheduler_chain_recursive_add (chain, peerelement);
-	}
+        gst_basic_scheduler_chain_recursive_add (chain, peerelement, remove);
       }
     }
   }
@@ -983,8 +998,11 @@ gst_basic_scheduler_reset (GstScheduler *sched)
   GList *elements = GST_BASIC_SCHEDULER_CAST (sched)->elements;
 
   while (elements) {
-    /* FIXME: wingo, do we need to destroy the cothreads here? */
-    GST_ELEMENT_THREADSTATE (elements->data) = NULL;
+    GstElement *element = GST_ELEMENT (elements->data);
+    if (GST_ELEMENT_THREADSTATE (element)) {
+      do_cothread_destroy (GST_ELEMENT_THREADSTATE (element));
+      GST_ELEMENT_THREADSTATE (element) = NULL;
+    }
     elements = g_list_next (elements);
   }
   
@@ -1038,8 +1056,10 @@ gst_basic_scheduler_remove_element (GstScheduler * sched, GstElement * element)
     chain = gst_basic_scheduler_find_chain (bsched, element);
 
     /* remove it from its chain */
-    gst_basic_scheduler_chain_remove_element (chain, element);
-
+    if (chain != NULL) {
+      gst_basic_scheduler_chain_remove_element (chain, element);
+    }
+    
     /* remove it from the list of elements */
     bsched->elements = g_list_remove (bsched->elements, element);
     bsched->num_elements--;
@@ -1112,7 +1132,7 @@ gst_basic_scheduler_unlock_element (GstScheduler * sched, GstElement * element)
     do_cothread_unlock (GST_ELEMENT_THREADSTATE (element));
 }
 
-static void
+static gboolean
 gst_basic_scheduler_yield (GstScheduler *sched, GstElement *element)
 {
   if (GST_ELEMENT_IS_COTHREAD_STOPPING (element)) {
@@ -1121,6 +1141,7 @@ gst_basic_scheduler_yield (GstScheduler *sched, GstElement *element)
     
     /* no need to do a pre_run, the cothread is stopping */
   }
+  return FALSE;
 }
 
 static gboolean
@@ -1152,7 +1173,7 @@ gst_basic_scheduler_error (GstScheduler *sched, GstElement *element)
 }
 
 static void
-gst_basic_scheduler_pad_connect (GstScheduler * sched, GstPad *srcpad, GstPad *sinkpad)
+gst_basic_scheduler_pad_link (GstScheduler * sched, GstPad *srcpad, GstPad *sinkpad)
 {
   GstElement *srcelement, *sinkelement;
   GstBasicScheduler *bsched = GST_BASIC_SCHEDULER (sched);
@@ -1162,7 +1183,7 @@ gst_basic_scheduler_pad_connect (GstScheduler * sched, GstPad *srcpad, GstPad *s
   sinkelement = GST_PAD_PARENT (sinkpad);
   g_return_if_fail (sinkelement != NULL);
 
-  GST_INFO (GST_CAT_SCHEDULING, "have pad connected callback on %s:%s to %s:%s",
+  GST_INFO (GST_CAT_SCHEDULING, "have pad linked callback on %s:%s to %s:%s",
 	    GST_DEBUG_PAD_NAME (srcpad), GST_DEBUG_PAD_NAME (sinkpad));
   GST_DEBUG (GST_CAT_SCHEDULING, "srcpad sched is %p, sinkpad sched is %p",
 	     GST_ELEMENT_SCHED (srcelement), GST_ELEMENT_SCHED (sinkelement));
@@ -1175,13 +1196,13 @@ gst_basic_scheduler_pad_connect (GstScheduler * sched, GstPad *srcpad, GstPad *s
 }
 
 static void
-gst_basic_scheduler_pad_disconnect (GstScheduler * sched, GstPad * srcpad, GstPad * sinkpad)
+gst_basic_scheduler_pad_unlink (GstScheduler * sched, GstPad * srcpad, GstPad * sinkpad)
 {
   GstElement *element1, *element2;
   GstSchedulerChain *chain1, *chain2;
   GstBasicScheduler *bsched = GST_BASIC_SCHEDULER (sched);
 
-  GST_INFO (GST_CAT_SCHEDULING, "disconnecting pads %s:%s and %s:%s",
+  GST_INFO (GST_CAT_SCHEDULING, "unlinking pads %s:%s and %s:%s",
 	    GST_DEBUG_PAD_NAME (srcpad), GST_DEBUG_PAD_NAME (sinkpad));
 
   /* we need to have the parent elements of each pad */
@@ -1196,6 +1217,12 @@ gst_basic_scheduler_pad_disconnect (GstScheduler * sched, GstPad * srcpad, GstPa
   chain1 = gst_basic_scheduler_find_chain (bsched, element1);
   chain2 = gst_basic_scheduler_find_chain (bsched, element2);
 
+/* FIXME: The old code still works in most cases, but does not deal with
+ * the problem of screwed up sched chains in some autoplugging cases.
+ * The new code has an infinite recursion bug during pipeline shutdown,
+ * which must be fixed before it can be enabled again.
+ */
+#if 1
   if (chain1 != chain2) {
     /* elements not in the same chain don't need to be separated */
     GST_INFO (GST_CAT_SCHEDULING, "elements not in the same chain");
@@ -1208,15 +1235,25 @@ gst_basic_scheduler_pad_disconnect (GstScheduler * sched, GstPad * srcpad, GstPa
 
     /* now create a new chain to hold element1 and build it from scratch */
     chain1 = gst_basic_scheduler_chain_new (bsched);
-    gst_basic_scheduler_chain_recursive_add (chain1, element1);
+    gst_basic_scheduler_chain_recursive_add (chain1, element1, FALSE);
   }
 
   /* check the other element to see if it landed in the newly created chain */
   if (gst_basic_scheduler_find_chain (bsched, element2) == NULL) {
     /* if not in chain, create chain and build from scratch */
     chain2 = gst_basic_scheduler_chain_new (bsched);
-    gst_basic_scheduler_chain_recursive_add (chain2, element2);
+    gst_basic_scheduler_chain_recursive_add (chain2, element2, FALSE);
   }
+
+#else
+
+  /* if they're both in the same chain, move second set of elements to a new chain */
+  if (chain1 && (chain1 == chain2)) {
+    GST_INFO (GST_CAT_SCHEDULING, "creating new chain for second element and peers");
+    chain2 = gst_basic_scheduler_chain_new (bsched);
+    gst_basic_scheduler_chain_recursive_add (chain2, element2, TRUE);
+  }
+#endif
 }
 
 static GstPad *
@@ -1247,7 +1284,8 @@ gst_basic_scheduler_pad_select (GstScheduler * sched, GList * padlist)
 
     do_element_switch (GST_PAD_PARENT (peer));
 
-    pad = GST_ELEMENT (GST_PAD_PARENT (pad))->select_pad;
+    /* FIXME disabled for now */
+    /* pad = GST_ELEMENT (GST_PAD_PARENT (pad))->select_pad;*/
 
     g_assert (pad != NULL);
   }
@@ -1256,12 +1294,8 @@ gst_basic_scheduler_pad_select (GstScheduler * sched, GList * padlist)
 
 static GstClockReturn
 gst_basic_scheduler_clock_wait (GstScheduler *sched, GstElement *element,
-				GstClock *clock, GstClockTime time, GstClockTimeDiff *jitter)
+				GstClockID id, GstClockTimeDiff *jitter)
 {
-  GstClockID id;
-  
-  id = gst_clock_new_single_shot_id (clock, time);
-
   return gst_clock_id_wait (id, jitter);
 }
 
