@@ -43,7 +43,7 @@ print_prop (GstPropsEntry *prop, gboolean showname, const gchar *pfx)
       g_print("Float range: %f - %f\n", min, max);
       break;
     }
-    case GST_PROPS_BOOL_TYPE:
+    case GST_PROPS_BOOLEAN_TYPE:
     {
       gboolean val;
       gst_props_entry_get_boolean (prop, &val);
@@ -150,6 +150,23 @@ print_formats (const GstFormat *formats)
   }
 }
 
+static void 
+print_query_types (const GstQueryType *types) 
+{
+  while (types && *types) {
+    const GstQueryTypeDefinition *definition;
+
+    definition = gst_query_type_get_details (*types);
+    if (definition)
+      g_print ("\t\t(%d):\t%s (%s)\n", *types,
+	       definition->nick, definition->description);
+    else
+      g_print ("\t\t(%d):\tUnknown query format\n", *types);
+
+    types++;
+  }
+}
+
 static void 
 print_event_masks (const GstEventMask *masks) 
 {
@@ -198,26 +215,6 @@ print_event_masks (const GstEventMask *masks)
   }
 }
 
-static void 
-print_query_types (const GstPadQueryType *types) 
-{
-  GType query_type;
-  GEnumClass *klass;
-
-  query_type = gst_pad_query_type_get_type();
-  klass = (GEnumClass *) g_type_class_ref (query_type);
-
-  while (types && *types) {
-    GEnumValue *value;
-    
-    value = g_enum_get_value (klass, *types);
-    
-    g_print ("\t\t(%d):\t%s (%s)\n", value->value, value->value_nick, value->value_name);
-    types++;
-  }
-}
-
-
 static void
 output_hierarchy (GType type, gint level, gint *maxlevel)
 {
@@ -418,14 +415,18 @@ print_element_properties (GstElement *element)
 	  if (flags)
 	    g_string_free (flags, TRUE);
 	}
-        else {
-          g_print ("%-23.23s Unknown type %ld \"%s\"", "",param->value_type, 
+	else if (G_IS_PARAM_SPEC_OBJECT (param)) {
+	  g_print("%-23.23s Object of type \"%s\"", "",
 			  g_type_name(param->value_type));
+        }
+	else {
+          g_print ("%-23.23s Unknown type %ld \"%s\"", "",param->value_type, 
+		  	g_type_name(param->value_type));
 	}
         break;
     }
     if (!readable) 
-      g_print ("Write only\n");
+      g_print (" Write only\n");
     else 
       g_print ("\n");
   }
@@ -567,22 +568,31 @@ print_element_info (GstElementFactory *factory)
   have_flags = FALSE;
 
   g_print ("\nClocking Interaction:\n");
-  if (element->setclockfunc) {
+  if (gst_element_requires_clock (element)) {
     g_print ("  element requires a clock\n");
     have_flags = TRUE;
   }
-  if (element->getclockfunc) {
+  if (gst_element_provides_clock (element)) {
     GstClock *clock;
 
     clock = gst_element_get_clock (element);
     if (clock)
       g_print ("  element provides a clock: %s\n", GST_OBJECT_NAME(clock));
+    else
+      g_print ("  element is supposed to provide a clock but returned NULL\n");
     have_flags = TRUE;
   }
   if (!have_flags) {
     g_print ("  none\n");
   }
 
+  g_print ("\nIndexing capabilities:\n");
+  if (gst_element_is_indexable (element)) {
+    g_print ("  element can do indexing\n");
+  }
+  else {
+    g_print ("  none\n");
+  }
 
   g_print ("\nPads:\n");
   if (element->numpads) {
@@ -634,9 +644,9 @@ print_element_info (GstElementFactory *factory)
 	print_query_types (gst_pad_get_query_types (GST_PAD (realpad)));
       }
 
-      if (realpad->intconnfunc != gst_pad_get_internal_connections_default)
+      if (realpad->intlinkfunc != gst_pad_get_internal_links_default)
         g_print ("      Has custom intconnfunc(): %s\n",
-	        GST_DEBUG_FUNCPTR_NAME(realpad->intconnfunc));
+	        GST_DEBUG_FUNCPTR_NAME(realpad->intlinkfunc));
 
       if (realpad->bufferpoolfunc)
         g_print ("      Has bufferpoolfunc(): %s\n",
@@ -730,14 +740,17 @@ print_element_info (GstElementFactory *factory)
           return_type = query->return_type;
           param_types = query->param_types;
 
-          g_print ("  \"%s\" :\t %s user_function (%s* object, \n", 
+          g_print ("  \"%s\" :\t %s user_function (%s* object", 
 	          query->signal_name, g_type_name (return_type),
 		  g_type_name (G_OBJECT_TYPE (element)));
 
           for (j = 0; j < n_params; j++) {
-            g_print ("    \t\t\t\t%s arg%d,\n", g_type_name (param_types[j]), j);
+            g_print (",\n    \t\t\t\t%s arg%d", g_type_name (param_types[j]), j);
           }
-          g_print ("    \t\t\t\tgpointer user_data);\n");
+	  if (k == 0)
+            g_print (",\n    \t\t\t\tgpointer user_data);\n");
+	  else
+            g_print (");\n");
 
 	  counted++;
 	}
@@ -801,6 +814,13 @@ print_element_list (void)
         g_print ("%s:  %s: %s\n", plugin->name, 
 	        GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
+      else if (GST_IS_INDEX_FACTORY (feature)) {
+        GstIndexFactory *factory;
+
+        factory = GST_INDEX_FACTORY (feature);
+        g_print ("%s:  %s: %s\n", plugin->name, 
+	        GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
+      }
       else if (GST_IS_TYPE_FACTORY (feature)) {
         GstTypeFactory *factory;
 
@@ -819,6 +839,14 @@ print_element_list (void)
         g_print ("%s:  %s: %s\n", plugin->name, 
 	        GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
+      else if (GST_IS_URI_HANDLER (feature)) {
+        GstURIHandler *handler;
+
+        handler = GST_URI_HANDLER (feature);
+        g_print ("%s:  %s: \"%s\" (%s) element \"%s\" property \"%s\"\n", plugin->name, 
+	        GST_PLUGIN_FEATURE_NAME (handler), handler->uri, handler->longdesc,
+		handler->element, handler->property);
+      }
       else {
         g_print ("%s:  %s (%s)\n", plugin->name, 
 	        GST_PLUGIN_FEATURE_NAME (feature), 
@@ -834,6 +862,13 @@ static void
 print_plugin_info (GstPlugin *plugin)
 {
   GList *features;
+  gint num_features = 0;
+  gint num_elements = 0;
+  gint num_autoplug = 0;
+  gint num_types = 0;
+  gint num_schedulers = 0;
+  gint num_indexes = 0;
+  gint num_other = 0;
   
   g_print ("Plugin Details:\n");
   g_print ("  Name:\t\t%s\n",    plugin->name);
@@ -854,12 +889,21 @@ print_plugin_info (GstPlugin *plugin)
       factory = GST_ELEMENT_FACTORY (feature);
       g_print ("  %s: %s\n", GST_OBJECT_NAME (factory),
 	      factory->details->longname);
+      num_elements++;
     }
     else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
       GstAutoplugFactory *factory;
 
       factory = GST_AUTOPLUG_FACTORY (feature);
       g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
+      num_autoplug++;
+    }
+    else if (GST_IS_INDEX_FACTORY (feature)) {
+      GstIndexFactory *factory;
+
+      factory = GST_INDEX_FACTORY (feature);
+      g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
+      num_indexes++;
     }
     else if (GST_IS_TYPE_FACTORY (feature)) {
       GstTypeFactory *factory;
@@ -870,21 +914,37 @@ print_plugin_info (GstPlugin *plugin)
       if (factory->typefindfunc)
         g_print ("      Has typefind function: %s\n", 
 	        GST_DEBUG_FUNCPTR_NAME (factory->typefindfunc));
+      num_types++;
     }
     else if (GST_IS_SCHEDULER_FACTORY (feature)) {
       GstSchedulerFactory *factory;
 
       factory = GST_SCHEDULER_FACTORY (feature);
       g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
+      num_schedulers++;
     }
     else {
       g_print ("  %s (%s)\n", gst_object_get_name (GST_OBJECT (feature)), 
 		             g_type_name (G_OBJECT_TYPE (feature)));
+      num_other++;
     }
-
-
+    num_features++;
     features = g_list_next (features);
   }
+  g_print ("\n  %d features:\n", num_features);
+  if (num_elements > 0)
+    g_print ("  +-- %d elements\n", num_elements);
+  if (num_autoplug > 0)
+    g_print ("  +-- %d autopluggers\n", num_autoplug);
+  if (num_types > 0)
+    g_print ("  +-- %d types\n", num_types);
+  if (num_schedulers > 0)
+    g_print ("  +-- %d schedulers\n", num_schedulers);
+  if (num_indexes > 0)
+    g_print ("  +-- %d indexes\n", num_indexes);
+  if (num_other > 0)
+    g_print ("  +-- %d other objects\n", num_other);
+  
   g_print ("\n");
 }
 
@@ -928,6 +988,36 @@ main (int argc, char *argv[])
       /* if there's a factory, print out the info */
       if (factory)
         return print_element_info (factory);
+      else {
+	 GstPluginFeature* feature;
+
+	 /* FIXME implement other pretty print function for these */
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_SCHEDULER_FACTORY);
+	 if (feature) {
+           g_print ("%s: a scheduler\n", argv[1]);
+	   return 0;
+	 }
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_INDEX_FACTORY);
+	 if (feature) {
+           g_print ("%s: an index\n", argv[1]);
+	   return 0;
+	 }
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_AUTOPLUG_FACTORY);
+	 if (feature) {
+           g_print ("%s: an autoplugger\n", argv[1]);
+	   return 0;
+	 }
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_TYPE_FACTORY);
+	 if (feature) {
+           g_print ("%s: an type\n", argv[1]);
+	   return 0;
+	 }
+	 feature = gst_registry_pool_find_feature (argv[1], GST_TYPE_URI_HANDLER);
+	 if (feature) {
+           g_print ("%s: an uri handler\n", argv[1]);
+	   return 0;
+	 }
+      }
     } else {
       /* strip the .so */
       so = strstr(argv[1],".so");
