@@ -17,9 +17,11 @@ idle_func (gpointer data)
   GTimeVal tfthen, tfnow;
   GstClockTimeDiff diff;
 
+  /*
   if (s_clock) {
-    //g_print ("%lld\n", gst_clock_get_time (s_clock));
+    g_print ("%lld\n", gst_clock_get_time (s_clock));
   }
+  */
 
   g_get_current_time (&tfthen);
   busy = gst_bin_iterate (GST_BIN (data));
@@ -114,6 +116,10 @@ fault_handler (int signum, siginfo_t *si, void *misc)
 {
   int spinning = TRUE;
 
+  /* dolphy has seen si == NULL;
+   * that doesn't seem like a good idea, does it ? */
+  g_assert (si);
+
   fault_restore ();
 
   if (si->si_signo == SIGSEGV) {
@@ -204,10 +210,10 @@ main(int argc, char *argv[])
 
   free (malloc (8)); /* -lefence */
 
+  gst_init_with_popt_table (&argc, &argv, options);
+
   if (!no_fault)
     fault_setup();
-
-  gst_init_with_popt_table (&argc, &argv, options);
   
   /* make a null-terminated version of argv */
   argvn = g_new0 (char*, argc);
